from datetime import datetime
import os

from discordpy_common import Log


# Global directories paths
_base_directories: list[str] = [
    './src',
    '.src/guilds',
]

# Guild directory paths
_guild_directories: list[str] = [
    './src/guilds/{guild_id}',
    './src/guilds/{guild_id}/logs',
    './src/guilds/{guild_id}/authorized',
    './src/guilds/{guild_id}/temp',
]

# Guild files paths
_guild_files: list[str] = [
    './src/guilds/{guild_id}/authorized/admin.txt',
    './src/guilds/{guild_id}/logs/{date}.txt',
    './src/guilds/{guild_id}/temp/temp.txt',
]


def base_architecture() -> None :

    # Create all directories if not exist
    for directory in _base_directories :
        if not os.path.exists(directory) :
            os.mkdir(directory)
            print(f'Created {directory}')
        else :
            print(f'{directory} already exists')
    print('Architecture created !')


def create_guild_pattern(guild_id: str) -> None :
    """
    Create files and directories used by a guild
    :param guild_id: `str` the guild id
    :return:
    """

    # Create all directories if not exist
    for directory in _guild_directories :
        # Replace with current guild id
        directory = directory.format(guild_id=guild_id)
        if not os.path.exists(directory) :
            os.mkdir(directory)

    # Get current datetime
    date: datetime = datetime.now()
    ymd_format: str = date.strftime('%Y-%m-%d')

    # Create all files if not exist
    for file in _guild_files :
        # Replace with current guild and date
        file = file.format(guild_id=guild_id, date=ymd_format)
        if not os.path.exists(file) :
            with open(file, 'w', encoding='UTF-8') as f :
                f.write('')

    # Write a message in logs
    with open(f'.src/guilds/{guild_id}/logs/{ymd_format}.txt', 'a+', encoding='UTF-8') as f :
        f.write(Log.GUILD_CREATED(guild_id=guild_id, date=date).message)



from motor.motor_asyncio import AsyncIOMotorClient, AsyncIOMotorDatabase
from pymongo.server_api import ServerApi

from discordpy_common import Log

from datetime import datetime
import asyncio


# Override the AsyncIOMotorClient class
class Client(AsyncIOMotorClient) :

    main_db: AsyncIOMotorDatabase | None

    # Initialize the client and ping the server
    def __init__(self, uri: str) :
        super().__init__(uri, server_api=ServerApi('1'))
        asyncio.run(self._ping())
        self.main_db = None

    async def _ping(self) :

        # Check if the connection is successful
        try :
            await self.admin.command('ping')
            print('Pinged your deployment. You successfully connected to MongoDB !')

        except Exception as e :
            raise Exception('Could not connect to MongoDB', e)

    async def create_guild_db(self, guild_id: str, owner_id: str) -> None :

        # Create the guild database
        guild_db = self[guild_id]
        if auth_coll := guild_db.get_collection('authorized') is None :
            auth_coll = await guild_db.create_collection('authorized')

        # Insert the owner in the authorized collection
        await auth_coll.insert_one({
            '_id': owner_id,
            'auth': ['owner']
        })

        # Create the logs collection
        if logs_coll := guild_db.get_collection('logs') is None :
            logs_coll = await guild_db.create_collection('logs')

        # Get the number of logs for the current day
        nb_logs_day: int = len(await logs_coll.find({
            '_id': {
                '$regex': datetime.now().strftime('%Y-%m-%d')
            }}).to_list(None))

        # Create the guild created log
        date: datetime = datetime.now()
        guild_log: Log.GUILD_CREATED = Log.GUILD_CREATED(guild_id=guild_id, date=date)

        # Insert the log in the logs collection
        await logs_coll.insert_one({
            '_id': f'{date.strftime('%Y-%m-%d')}_{nb_logs_day}',
            'type': guild_log.log_type,
            'message': guild_log.message,
        })



import discord as ds
from discord.ext import commands

from enum import Enum
import asyncio
import os

from discordpy_common import (
    Client as mongoClient,
    base_architecture,
)

from motor.motor_asyncio import AsyncIOMotorDatabase


# Type aliases
type DBClient = mongoClient
type DB = AsyncIOMotorDatabase


# Intents
class Intents(ds.Intents) :
    pass


# Database types / Storage types
class TypeDB(Enum) :
    """
    Enum class that defines the different types of databases
    """
    FILE = 1
    MONGO = 2
    ...


# Override the Bot class
class Bot(commands.Bot) :

    _token: str
    typeDB: TypeDB
    client: DBClient
    db_name: str

    def __init__(
            self,
            *,
            token: str,
            prefix: str,
            desc: str,
            intents: Intents | None = Intents.all(),
            typeDB: TypeDB,
            cogs_dir: str | None = None,
            remove_help: bool = True,
            connection_string: str | None = None,
            db_name: str | None = None,
            **kwargs,
    ) -> None :

        # Initialize the bot
        super().__init__(command_prefix=prefix, description=desc, intents=intents, **kwargs)
        self._token = token
        self.db = typeDB
        self.db_name = db_name

        # Connect to the database
        if self.db != TypeDB.FILE :
            if connection_string is None :
                raise ValueError('You must provide a connection string for the database')

            self._connect_to_db(connection_string)

        # Create file architecture
        else :
            base_architecture()

        # Remove the help command
        self.remove_command('help') if remove_help else None
        # Start the bot
        asyncio.run(self._start_bot(token, cogs_dir))

    async def _load(self, cogs_dir: str) -> None :

        # Verify if the directory exists
        if not os.path.exists(cogs_dir) :
            raise FileNotFoundError(f'{cogs_dir} does not exist')

        # Load the cogs
        for file_name in os.listdir(cogs_dir) :
            if file_name.endswith('.py') :
                await self.load_extension(
                    f'{cogs_dir}.{file_name[:-3]}'
                    .replace('/', '.')
                    .replace('\\', '.')
                )
        print('Cogs loaded !')

    async def _start_bot(self, token: str, cog_dir: str | None = './cogs') -> None :

        # Load the cogs and start the bot
        if cog_dir is not None :
            await self._load(cog_dir)
        await super().start(token)

    def _connect_to_db(self, connection_string: str) -> None :

        # Define the connection modes
        connect_modes: dict[TypeDB, callable] = {
            TypeDB.MONGO : self._mongodb_connect,
        }
        # Connect to the database
        return connect_modes[self.typeDB](connection_string)

    def _mongodb_connect(self, connection_string: str) -> None :

        # Create client
        self.client: mongoClient = mongoClient(connection_string)

        # Connect to the default database
        if self.db_name is None :
            self.client.main_db = self.client.get_default_database()

            if self.client.main_db is None :
                raise ValueError('No database found')
            print(f'Connected to the default database: {self.client.main_db.name}')
            return

        # Connect to the specified database
        self.client.main_db = self.client.get_database(self.db_name)

        if self.client.main_db is None :
            raise ValueError(f'No database found with name {self.db_name}')
        print(f'Connected to the database: {self.client.main_db.name}')



from datetime import datetime
from enum import Enum


class LogType(Enum) :

    INFO = '[INFO]'
    ERROR = '[ERROR]'


class _GuildCreated :

    log_type: LogType
    date: str
    message: str

    def __init__(self, guild_id: str, date: datetime) -> None :
        log_type = LogType.INFO
        date = date.strftime('[%Y-%m-%d] [%H-%M-%S]')
        message = f'{date} [{log_type.value()}] Guild {guild_id} created. \n'


class Log(Enum) :

    GUILD_CREATED = _GuildCreated

    # Create a log
    def __call__(self, *args, **kwargs) :
        return self.value(*args, **kwargs)



from setuptools import setup, find_packages

setup(
    name='discordpy-common',
    version='0.2.0',
    packages=find_packages(),
    install_requires=[
        'discord.py',
        'motor',
        'pymongo',
    ],
    python_requires='>=3.12',
    author='pyPoul',
    license='MIT',
    description='Common tools for discord.py bots',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/pyPoul/discordpy-common',
)



import discord as ds
from discord.ext import commands

from collections.abc import Callable
from typing import Any
from enum import Enum

from _discord.bot import TypeDB

from motor.motor_asyncio import (
    AsyncIOMotorClient,
    AsyncIOMotorDatabase,
)


class AuthType(Enum) :
    ADMIN = 'admin'
    MODERATOR = 'moderator'
    PREMIUM = 'premium'
    USER = 'user'
    ...


# Verify in files
async def _file_auth(
        *,
        auth: AuthType,
        guild_id: str,
        member_id: str,
        **_
) -> bool :
    ...


# Verify in MongoDB
async def _mongo_auth(
        *,
        client: AsyncIOMotorClient,
        guild_id: str,
        member_id: str,
        auth: list[AuthType],
        **_
) -> bool :

    # Get the guild database
    guild_db: AsyncIOMotorDatabase = client.get_database(guild_id)
    if guild_db is None :
        return False

    # Get the authorized collection
    auth_collection = guild_db.get_collection('authorized')
    if auth_collection is None :
        return False

    # Check if the user or role is authorized
    user = await auth_collection.find_one({
        '_id': member_id,
        'auth': {
            '$in': [a.value for a in auth] + ['owner']
        }
    })
    if user is None :
        return False

    return True


# auth_only decorator
def auth_only(func: Callable[commands.Cog, ds.Interaction, tuple[Any, ...], dict[str, Any]], *auth: AuthType | None) -> Callable or None :
    """
    A decorator that checks if the user is authorized to execute the command
    :param func: `Callable` the command to execute
    :param auth: `AuthType` the authorization type. Default: `AuthType.ADMIN`
    :return:
    """

    # Define the different authentication modes
    auth_modes: dict[TypeDB, Callable] = {
        TypeDB.FILE : _file_auth,
        TypeDB.MONGO : _mongo_auth,
    }

    # Default auth type
    if auth is None :
        auth = [AuthType.ADMIN]

    async def wrapper(cog: commands.Cog, interaction: ds.Interaction, *args, **kwargs) -> Callable or None :

        # Verify if the user is authorized
        if not await auth_modes[cog.bot.typeDB](
                user_id=interaction.user.id.__str__(),
                guild_id=interaction.guild_id.__str__(),
                client=cog.bot.client,
                auth=auth,
        ) :
            await interaction.response.send_message('You are not authorized to do this !', ephemeral=True)
            return

        # Execute the command
        return func(cog, interaction, *args, **kwargs)

    return wrapper
